@@ -5,7 +5,7 @@ set -e
 WINE_DIR=$1
 
 if test -z $WINE_DIR; then
-    echo "Usage: wine-impot.sh wine_dir"
+    echo "Usage: wine-import.sh wine_dir"
     exit 1
 fi
 
@@ -43,75 +43,35 @@ import_idl() {
 }
 
 # headers
-for f in \
-	corerror.h \
-	mscat.h \
-	propkey.h \
-	propkeydef.h \
-	rpcsal.h \
-	t2embapi.h \
-	uiautomation.h \
-	uiautomationcoreapi.h \
-	winhttp.h \
-	winineti.h; do
-    import_header $f include
-done
-
-# IDLs
-for f in \
-	ctfutb \
-	devicetopology \
-	downloadmgr \
-	drmexternals \
-	endpointvolume \
-	fusion \
-	icftypes \
-	mmdeviceapi \
-	mscoree \
-	msctf \
-	netfw \
-	netlistmgr \
-	objectarray \
-	optary \
-	taskschd \
-	uiautomationclient \
-	uiautomationcore \
-	urlhist \
-	wmdrmsdk \
-	wmsbuffer \
-	wmsdkidl \
-	wpcapi \
-	xmllite; do
-    import_idl $f.idl include
-done
-
-# DirectX headers
 for f in \
 	amaudio.h \
 	audevcod.h \
-	d3d.h \
+	corerror.h \
+	d2dbasetypes.h \
 	d3d10_1shader.h \
 	d3d10effect.h \
 	d3d10misc.h \
 	d3d10shader.h \
 	d3d11shader.h \
-	d3d8.h \
 	d3d8caps.h \
+	d3d8.h \
 	d3d8types.h \
-	d3d9.h \
 	d3d9caps.h \
+	d3d9.h \
 	d3d9types.h \
 	d3dcaps.h \
 	d3dcompiler.h \
+	d3d.h \
 	d3dhal.h \
-	d3drm.h \
 	d3drmdef.h \
+	d3drm.h \
 	d3drmobj.h \
 	d3dtypes.h \
-	d3dx9.h \
+	d3dvec.inl \
 	d3dx9anim.h \
 	d3dx9core.h \
 	d3dx9effect.h \
+	d3dx9.h \
 	d3dx9math.h \
 	d3dx9math.inl \
 	d3dx9mesh.h \
@@ -119,7 +79,7 @@ for f in \
 	d3dx9shape.h \
 	d3dx9tex.h \
 	d3dx9xof.h \
-	d3dvec.inl \
+	devpkey.h \
 	dls1.h \
 	dls2.h \
 	dmerror.h \
@@ -133,58 +93,141 @@ for f in \
 	dmusici.h \
 	dmusics.h \
 	dpaddr.h \
-	dplay.h \
 	dplay8.h \
-	dplobby.h \
+	dplay.h \
 	dplobby8.h \
+	dplobby.h \
 	dpnathlp.h \
+	dsconf.h \
 	dshow.h \
 	dvdmedia.h \
 	dxdiag.h \
 	dxerr8.h \
 	dxerr9.h \
 	dxfile.h \
+	dxva.h \
 	errors.h \
 	evcode.h \
+	exdispid.h \
+	fontsub.h \
 	mediaerr.h \
+	mscat.h \
+	propkeydef.h \
+	propkey.h \
+	propvarutil.h \
+	rpcsal.h \
+	t2embapi.h \
+	uiautomationcoreapi.h \
+	uiautomation.h \
 	vfwmsgs.h \
+	winhttp.h \
+	winineti.h \
+	wmpids.h \
+	xapofx.h \
 	xinput.h; do
-    import_header $f direct-x/include
+    import_header $f include
 done
 
-# DirectX IDLs
+# IDLs
 for f in \
 	amstream \
 	amvideo \
+	asyncinfo \
 	austream \
+	bits \
+	bits1_5 \
+	bits2_0 \
+	bits2_5 \
+	bits3_0 \
+	bits5_0 \
+	ctfutb \
 	d3d10 \
 	d3d10_1 \
 	d3d10sdklayers \
 	d3d11 \
+	d3d11on12 \
 	d3d11_1 \
 	d3d11_2 \
 	d3d11_3 \
 	d3d11_4 \
 	d3d11sdklayers \
+	d3d12 \
+	d3d12sdklayers \
+	d3d12shader \
 	d3dcommon \
+	dcommon \
 	ddstream \
+	devicetopology \
+	directmanipulation \
+	downloadmgr \
+	drmexternals \
+	dvdif \
+	dwrite \
+	dwrite_1 \
+	dwrite_2 \
+	dwrite_3 \
 	dxgi \
 	dxgi1_2 \
 	dxgi1_3 \
 	dxgi1_4 \
 	dxgi1_5 \
 	dxgi1_6 \
-	mediaobj \
 	dxgicommon \
+	dxgidebug \
 	dxgiformat \
 	dxgitype \
+	dxva2api \
+	endpointvolume \
+	eventtoken \
+	evr9 \
+	exdisp \
+	fusion \
+	icftypes \
+	mediaobj \
+	mfreadwrite \
+	mmdeviceapi \
 	mmstream \
-	qedit; do
-    import_idl $f.idl direct-x/include
+	mscoree \
+	msctf \
+	mulres \
+	netfw \
+	netlistmgr \
+	objectarray \
+	optary \
+	proofofpossessioncookieinfo \
+	qedit \
+	relogger \
+	rtworkq \
+	spatialaudioclient \
+	taskschd \
+	thumbcache \
+	uiautomationclient \
+	uiautomationcore \
+	urlhist \
+	vmr9 \
+	wincodec \
+	wincodecsdk \
+	windowscontracts \
+	windows.foundation \
+	windows.foundation.collections \
+	windows.media \
+	windows.media.speechsynthesis \
+	wmdrmsdk \
+	wmp \
+	wmprealestate \
+	wmpservices \
+	wmsbuffer \
+	wmsdkidl \
+	wmsecure \
+	wpcapi \
+	wuapi \
+	xapo \
+	xaudio2 \
+	xaudio2fx \
+	xmllite; do
+    import_idl $f.idl include
 done
 
 echo Import complete. You need to update headers generated from IDL files now:
-echo $ ./configure --with-widl --host=i686-w64-mingw32
-echo $ make
-echo $ make distclean
+echo '$ ./configure --with-widl --host=i686-w64-mingw32 && make && make distclean'
 
