@@ -50,6 +50,7 @@ static int is_integer_type(const type_t *type)
         case TYPE_BASIC_INT64:
         case TYPE_BASIC_INT:
         case TYPE_BASIC_INT3264:
+        case TYPE_BASIC_LONG:
         case TYPE_BASIC_CHAR:
         case TYPE_BASIC_HYPER:
         case TYPE_BASIC_BYTE:
@@ -82,6 +83,7 @@ static int is_signed_integer_type(const type_t *type)
         case TYPE_BASIC_INT64:
         case TYPE_BASIC_INT:
         case TYPE_BASIC_INT3264:
+        case TYPE_BASIC_LONG:
             return type_basic_get_sign(type) < 0;
         case TYPE_BASIC_CHAR:
             return TRUE;
@@ -192,15 +194,15 @@ expr_t *make_exprt(enum expr_type type, var_t *var, expr_t *expr)
     expr_t *e;
     type_t *tref;
 
-    if (var->stgclass != STG_NONE && var->stgclass != STG_REGISTER)
+    if (var->declspec.stgclass != STG_NONE && var->declspec.stgclass != STG_REGISTER)
         error_loc("invalid storage class for type expression\n");
 
-    tref = var->type;
+    tref = var->declspec.type;
 
     e = xmalloc(sizeof(expr_t));
     e->type = type;
     e->ref = expr;
-    e->u.tref = tref;
+    e->u.tref = var->declspec;
     e->is_const = FALSE;
     if (type == EXPR_SIZEOF)
     {
@@ -222,8 +224,8 @@ expr_t *make_exprt(enum expr_type type, var_t *var, expr_t *expr)
             e->is_const = TRUE;
             if (is_signed_integer_type(tref))
             {
-                cast_mask = (1 << (cast_type_bits - 1)) - 1;
-                if (expr->cval & (1 << (cast_type_bits - 1)))
+                cast_mask = (1u << (cast_type_bits - 1)) - 1;
+                if (expr->cval & (1u << (cast_type_bits - 1)))
                     e->cval = -((-expr->cval) & cast_mask);
                 else
                     e->cval = expr->cval & cast_mask;
@@ -231,8 +233,8 @@ expr_t *make_exprt(enum expr_type type, var_t *var, expr_t *expr)
             else
             {
                 /* calculate ((1 << cast_type_bits) - 1) avoiding overflow */
-                cast_mask = ((1 << (cast_type_bits - 1)) - 1) |
-                            1 << (cast_type_bits - 1);
+                cast_mask = ((1u << (cast_type_bits - 1)) - 1) |
+                            1u << (cast_type_bits - 1);
                 e->cval = expr->cval & cast_mask;
             }
         }
@@ -460,6 +462,11 @@ static type_t *find_identifier(const char *identifier, const type_t *cont_type,
         case TYPE_POINTER:
         case TYPE_ARRAY:
         case TYPE_BITFIELD:
+        case TYPE_APICONTRACT:
+        case TYPE_RUNTIMECLASS:
+        case TYPE_PARAMETERIZED_TYPE:
+        case TYPE_PARAMETER:
+        case TYPE_DELEGATE:
             /* nothing to do */
             break;
         case TYPE_ALIAS:
@@ -472,7 +479,7 @@ static type_t *find_identifier(const char *identifier, const type_t *cont_type,
     if (fields) LIST_FOR_EACH_ENTRY( field, fields, const var_t, entry )
         if (field->name && !strcmp(identifier, field->name))
         {
-            type = field->type;
+            type = field->declspec.type;
             *found_in_cont_type = 1;
             break;
         }
@@ -480,7 +487,7 @@ static type_t *find_identifier(const char *identifier, const type_t *cont_type,
     if (!type)
     {
         var_t *const_var = find_const(identifier, 0);
-        if (const_var) type = const_var->type;
+        if (const_var) type = const_var->declspec.type;
     }
 
     return type;
@@ -519,11 +526,11 @@ static struct expression_type resolve_expression(const struct expr_loc *expr_loc
         break;
     case EXPR_STRLIT:
         result.is_temporary = TRUE;
-        result.type = type_new_pointer(RPC_FC_UP, type_new_int(TYPE_BASIC_CHAR, 0), NULL);
+        result.type = type_new_pointer(type_new_int(TYPE_BASIC_CHAR, 0));
         break;
     case EXPR_WSTRLIT:
         result.is_temporary = TRUE;
-        result.type = type_new_pointer(RPC_FC_UP, type_new_int(TYPE_BASIC_WCHAR, 0), NULL);
+        result.type = type_new_pointer(type_new_int(TYPE_BASIC_WCHAR, 0));
         break;
     case EXPR_CHARCONST:
         result.is_temporary = TRUE;
@@ -571,17 +578,17 @@ static struct expression_type resolve_expression(const struct expr_loc *expr_loc
             error_loc_info(&expr_loc->v->loc_info, "address-of operator applied to non-variable type in expression%s%s\n",
                            expr_loc->attr ? " for attribute " : "",
                            expr_loc->attr ? expr_loc->attr : "");
-            result.is_variable = FALSE;
+        result.is_variable = FALSE;
         result.is_temporary = TRUE;
-        result.type = type_new_pointer(RPC_FC_UP, result.type, NULL);
+        result.type = type_new_pointer(result.type);
         break;
     case EXPR_PPTR:
         result = resolve_expression(expr_loc, cont_type, e->ref);
         if (result.type && is_ptr(result.type))
-            result.type = type_pointer_get_ref(result.type);
+            result.type = type_pointer_get_ref_type(result.type);
         else if(result.type && is_array(result.type)
                             && type_array_is_decl_as_ptr(result.type))
-            result.type = type_array_get_element(result.type);
+            result.type = type_array_get_element_type(result.type);
         else
             error_loc_info(&expr_loc->v->loc_info, "dereference operator applied to non-pointer type in expression%s%s\n",
                            expr_loc->attr ? " for attribute " : "",
@@ -589,7 +596,7 @@ static struct expression_type resolve_expression(const struct expr_loc *expr_loc
         break;
     case EXPR_CAST:
         result = resolve_expression(expr_loc, cont_type, e->ref);
-        result.type = e->u.tref;
+        result.type = e->u.tref.type;
         break;
     case EXPR_SIZEOF:
         result.is_temporary = FALSE;
@@ -663,7 +670,7 @@ static struct expression_type resolve_expression(const struct expr_loc *expr_loc
         if (result.type && is_array(result.type))
         {
             struct expression_type index_result;
-            result.type = type_array_get_element(result.type);
+            result.type = type_array_get_element_type(result.type);
             index_result = resolve_expression(expr_loc, cont_type /* FIXME */, e->u.ext);
             if (!index_result.type || !is_integer_type(index_result.type))
                 error_loc_info(&expr_loc->v->loc_info, "array subscript not of integral type in expression%s%s\n",
@@ -757,13 +764,13 @@ void write_expr(FILE *h, const expr_t *e, int brackets,
         break;
     case EXPR_CAST:
         fprintf(h, "(");
-        write_type_decl(h, e->u.tref, NULL, "");
+        write_type_decl(h, &e->u.tref, NULL);
         fprintf(h, ")");
         write_expr(h, e->ref, 1, toplevel, toplevel_prefix, cont_type, local_var_prefix);
         break;
     case EXPR_SIZEOF:
         fprintf(h, "sizeof(");
-        write_type_decl(h, e->u.tref, NULL, "");
+        write_type_decl(h, &e->u.tref, NULL);
         fprintf(h, ")");
         break;
     case EXPR_SHL:
@@ -915,7 +922,7 @@ int compare_expr(const expr_t *a, const expr_t *b)
                 return ret;
             return compare_expr(a->u.ext, b->u.ext);
         case EXPR_CAST:
-            ret = compare_type(a->u.tref, b->u.tref);
+            ret = compare_type(a->u.tref.type, b->u.tref.type);
             if (ret != 0)
                 return ret;
             /* Fall through.  */
@@ -927,7 +934,7 @@ int compare_expr(const expr_t *a, const expr_t *b)
         case EXPR_POS:
             return compare_expr(a->ref, b->ref);
         case EXPR_SIZEOF:
-            return compare_type(a->u.tref, b->u.tref);
+            return compare_type(a->u.tref.type, b->u.tref.type);
         case EXPR_VOID:
             return 0;
     }
