@@ -20,26 +20,18 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
-#include "wine/wpp.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
 #include <string.h>
 #include <ctype.h>
 
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
-
+#include "widl.h"
 #include "windef.h"
 #include "winbase.h"
-
-#include "widl.h"
 #include "utils.h"
+#include "wpp_private.h"
 #include "parser.h"
 #include "header.h"
 #include "typelib.h"
@@ -47,7 +39,6 @@
 #include "typelib_struct.h"
 #include "typetree.h"
 
-static typelib_t *typelib;
 
 /* List of oleauto types that should be recognized by name.
  * (most of) these seem to be intrinsic types in mktyplib.
@@ -68,7 +59,6 @@ static const struct oatype {
   {"VARIANT",       VT_VARIANT},
   {"VARIANT_BOOL",  VT_BOOL}
 };
-#define NTYPES (sizeof(oatypes)/sizeof(oatypes[0]))
 #define KWP(p) ((const struct oatype *)(p))
 
 static int kw_cmp_func(const void *s1, const void *s2)
@@ -83,11 +73,11 @@ static unsigned short builtin_vt(const type_t *t)
   const struct oatype *kwp;
   key.kw = kw;
 #ifdef KW_BSEARCH
-  kwp = bsearch(&key, oatypes, NTYPES, sizeof(oatypes[0]), kw_cmp_func);
+  kwp = bsearch(&key, oatypes, ARRAY_SIZE(oatypes), sizeof(oatypes[0]), kw_cmp_func);
 #else
   {
     unsigned int i;
-    for (kwp=NULL, i=0; i < NTYPES; i++)
+    for (kwp = NULL, i = 0; i < ARRAY_SIZE(oatypes); i++)
       if (!kw_cmp_func(&key, &oatypes[i])) {
         kwp = &oatypes[i];
         break;
@@ -101,9 +91,9 @@ static unsigned short builtin_vt(const type_t *t)
   {
     const type_t *elem_type;
     if (is_array(t))
-      elem_type = type_array_get_element(t);
+      elem_type = type_array_get_element_type(t);
     else
-      elem_type = type_pointer_get_ref(t);
+      elem_type = type_pointer_get_ref_type(t);
     if (type_get_type(elem_type) == TYPE_BASIC)
     {
       switch (type_basic_get_type(elem_type))
@@ -133,7 +123,8 @@ unsigned short get_type_vt(type_t *t)
     if (vt) return vt;
   }
 
-  if (type_is_alias(t) && is_attr(t->attrs, ATTR_PUBLIC))
+  if (type_is_alias(t) &&
+        (is_attr(t->attrs, ATTR_PUBLIC) || is_attr(t->attrs, ATTR_WIREMARSHAL)))
     return VT_USERDEFINED;
 
   switch (type_get_type(t)) {
@@ -160,6 +151,7 @@ unsigned short get_type_vt(type_t *t)
       else
         return VT_INT;
     case TYPE_BASIC_INT32:
+    case TYPE_BASIC_LONG:
     case TYPE_BASIC_ERROR_STATUS_T:
       if (type_basic_get_sign(t) > 0)
         return VT_UI4;
@@ -172,7 +164,7 @@ unsigned short get_type_vt(type_t *t)
       else
         return VT_I8;
     case TYPE_BASIC_INT3264:
-      if (typelib_kind == SYS_WIN64)
+      if (pointer_size == 8)
       {
         if (type_basic_get_sign(t) > 0)
           return VT_UI8;
@@ -201,12 +193,12 @@ unsigned short get_type_vt(type_t *t)
   case TYPE_ARRAY:
     if (type_array_is_decl_as_ptr(t))
     {
-      if (match(type_array_get_element(t)->name, "SAFEARRAY"))
+      if (match(type_array_get_element_type(t)->name, "SAFEARRAY"))
         return VT_SAFEARRAY;
+      return VT_PTR;
     }
     else
-      error("get_type_vt: array types not supported\n");
-    return VT_PTR;
+      return VT_CARRAY;
 
   case TYPE_INTERFACE:
     if(match(t->name, "IUnknown"))
@@ -221,13 +213,18 @@ unsigned short get_type_vt(type_t *t)
   case TYPE_MODULE:
   case TYPE_UNION:
   case TYPE_ENCAPSULATED_UNION:
+  case TYPE_RUNTIMECLASS:
+  case TYPE_DELEGATE:
     return VT_USERDEFINED;
 
   case TYPE_VOID:
     return VT_VOID;
 
   case TYPE_ALIAS:
-    /* aliases should be filtered out by the type_get_type call above */
+  case TYPE_APICONTRACT:
+  case TYPE_PARAMETERIZED_TYPE:
+  case TYPE_PARAMETER:
+    /* not supposed to be here */
     assert(0);
     break;
 
@@ -242,124 +239,147 @@ unsigned short get_type_vt(type_t *t)
   return 0;
 }
 
-void start_typelib(typelib_t *typelib_type)
+static void msft_read_guid(void *data, MSFT_SegDir *segdir, int offset, struct uuid *guid)
 {
-    if (!do_typelib) return;
-    typelib = typelib_type;
+    memcpy( guid, (char *)data + segdir->pGuidTab.offset + offset, sizeof(*guid) );
 }
 
-void end_typelib(void)
+static void read_msft_importlib(importlib_t *importlib, void *data, unsigned int size)
 {
-    if (!typelib) return;
-
-    create_msft_typelib(typelib);
-}
-
-static void tlb_read(int fd, void *buf, int count)
-{
-    if(read(fd, buf, count) < count)
-        error("error while reading importlib.\n");
-}
-
-static void tlb_lseek(int fd, off_t offset)
-{
-    if(lseek(fd, offset, SEEK_SET) == -1)
-        error("lseek failed\n");
-}
-
-static void msft_read_guid(int fd, MSFT_SegDir *segdir, int offset, GUID *guid)
-{
-    tlb_lseek(fd, segdir->pGuidTab.offset+offset);
-    tlb_read(fd, guid, sizeof(GUID));
-}
-
-static void read_msft_importlib(importlib_t *importlib, int fd)
-{
-    MSFT_Header header;
-    MSFT_SegDir segdir;
+    MSFT_Header *header = data;
+    MSFT_SegDir *segdir;
     int *typeinfo_offs;
     int i;
 
     importlib->allocated = 0;
+    importlib->version = header->version;
 
-    tlb_lseek(fd, 0);
-    tlb_read(fd, &header, sizeof(header));
+    typeinfo_offs = (int *)(header + 1);
+    segdir = (MSFT_SegDir *)(typeinfo_offs + header->nrtypeinfos);
 
-    importlib->version = header.version;
+    msft_read_guid(data, segdir, header->posguid, &importlib->guid);
 
-    typeinfo_offs = xmalloc(header.nrtypeinfos*sizeof(INT));
-    tlb_read(fd, typeinfo_offs, header.nrtypeinfos*sizeof(INT));
-    tlb_read(fd, &segdir, sizeof(segdir));
-
-    msft_read_guid(fd, &segdir, header.posguid, &importlib->guid);
-
-    importlib->ntypeinfos = header.nrtypeinfos;
+    importlib->ntypeinfos = header->nrtypeinfos;
     importlib->importinfos = xmalloc(importlib->ntypeinfos*sizeof(importinfo_t));
 
     for(i=0; i < importlib->ntypeinfos; i++) {
-        MSFT_TypeInfoBase base;
-        MSFT_NameIntro nameintro;
+        MSFT_TypeInfoBase *base = (MSFT_TypeInfoBase *)((char *)(segdir + 1) + typeinfo_offs[i]);
+        MSFT_NameIntro *nameintro;
         int len;
 
-        tlb_lseek(fd, sizeof(MSFT_Header) + header.nrtypeinfos*sizeof(INT) + sizeof(MSFT_SegDir)
-                 + typeinfo_offs[i]);
-        tlb_read(fd, &base, sizeof(base));
-
         importlib->importinfos[i].importlib = importlib;
-        importlib->importinfos[i].flags = (base.typekind&0xf)<<24;
+        importlib->importinfos[i].flags = (base->typekind & 0xf)<<24;
         importlib->importinfos[i].offset = -1;
         importlib->importinfos[i].id = i;
 
-        if(base.posguid != -1) {
+        if(base->posguid != -1) {
             importlib->importinfos[i].flags |= MSFT_IMPINFO_OFFSET_IS_GUID;
-            msft_read_guid(fd, &segdir, base.posguid, &importlib->importinfos[i].guid);
+            msft_read_guid(data, segdir, base->posguid, &importlib->importinfos[i].guid);
         }
         else memset( &importlib->importinfos[i].guid, 0, sizeof(importlib->importinfos[i].guid));
 
-        tlb_lseek(fd, segdir.pNametab.offset + base.NameOffset);
-        tlb_read(fd, &nameintro, sizeof(nameintro));
-
-        len = nameintro.namelen & 0xff;
+        nameintro = (MSFT_NameIntro *)((char *)data + segdir->pNametab.offset + base->NameOffset);
 
+        len = nameintro->namelen & 0xff;
         importlib->importinfos[i].name = xmalloc(len+1);
-        tlb_read(fd, importlib->importinfos[i].name, len);
+        memcpy( importlib->importinfos[i].name, nameintro + 1, len );
         importlib->importinfos[i].name[len] = 0;
     }
+}
 
-    free(typeinfo_offs);
+static unsigned int rva_to_va( const IMAGE_NT_HEADERS32 *nt, unsigned int rva )
+{
+    IMAGE_SECTION_HEADER *sec = (IMAGE_SECTION_HEADER *)((char *)&nt->OptionalHeader + nt->FileHeader.SizeOfOptionalHeader);
+    unsigned int i;
+
+    for (i = 0; i < nt->FileHeader.NumberOfSections; i++, sec++)
+        if (sec->VirtualAddress <= rva && sec->VirtualAddress + sec->Misc.VirtualSize > rva)
+            return sec->PointerToRawData + (rva - sec->VirtualAddress);
+    error( "no PE section found for addr %x\n", rva );
+}
+
+static void read_pe_importlib(importlib_t *importlib, void *data, unsigned int size)
+{
+    IMAGE_DOS_HEADER *dos = data;
+    IMAGE_NT_HEADERS32 *nt;
+    IMAGE_DATA_DIRECTORY *dir;
+    IMAGE_RESOURCE_DIRECTORY *root, *resdir;
+    IMAGE_RESOURCE_DIRECTORY_ENTRY *entry;
+    IMAGE_RESOURCE_DATA_ENTRY *resdata;
+    void *ptr;
+    unsigned int i, va;
+
+    if (dos->e_lfanew < sizeof(*dos) || dos->e_lfanew >= size) error( "not a PE file\n" );
+    nt = (IMAGE_NT_HEADERS32 *)((char *)data + dos->e_lfanew);
+    if (nt->Signature != IMAGE_NT_SIGNATURE) error( "not a PE file\n" );
+    if ((char *)&nt->OptionalHeader + nt->FileHeader.SizeOfOptionalHeader > (char *)data + size)
+        error( "invalid PE file\n" );
+    switch (nt->OptionalHeader.Magic)
+    {
+    case IMAGE_NT_OPTIONAL_HDR32_MAGIC:
+        dir = &nt->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_RESOURCE];
+        break;
+    case IMAGE_NT_OPTIONAL_HDR64_MAGIC:
+        dir = &((IMAGE_NT_HEADERS64 *)nt)->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_RESOURCE];
+        break;
+    default:
+        error( "invalid PE file\n" );
+    }
+    if (!dir->VirtualAddress || !dir->Size) error( "resource not found in PE file\n" );
+    va = rva_to_va( nt, dir->VirtualAddress );
+    if (va + dir->Size > size) error( "invalid resource data in PE file\n" );
+    root = resdir = (IMAGE_RESOURCE_DIRECTORY *)((char *)data + va );
+    entry = (IMAGE_RESOURCE_DIRECTORY_ENTRY *)(resdir + 1);
+    for (i = 0; i < resdir->NumberOfNamedEntries; i++, entry++)
+    {
+        static const WCHAR typelibW[] = {'T','Y','P','E','L','I','B'};
+        WCHAR *name = (WCHAR *)((char *)root + entry->NameOffset);
+        if (name[0] != ARRAY_SIZE(typelibW)) continue;
+        if (!memcmp( name + 1, typelibW, sizeof(typelibW) )) break;
+    }
+    if (i == resdir->NumberOfNamedEntries) error( "typelib resource not found in PE file\n" );
+    while (entry->DataIsDirectory)
+    {
+        resdir = (IMAGE_RESOURCE_DIRECTORY *)((char *)root + entry->OffsetToDirectory);
+        entry = (IMAGE_RESOURCE_DIRECTORY_ENTRY *)(resdir + 1);
+    }
+    resdata = (IMAGE_RESOURCE_DATA_ENTRY *)((char *)root + entry->OffsetToData);
+    ptr = (char *)data + rva_to_va( nt, resdata->OffsetToData );
+    if (memcmp( ptr, "MSFT", 4 )) error( "invalid typelib found in PE file\n" );
+    read_msft_importlib( importlib, ptr, resdata->Size );
 }
 
 static void read_importlib(importlib_t *importlib)
 {
-    int fd;
-    INT magic;
-    char *file_name;
+    int fd, size;
+    void *data;
 
-    file_name = wpp_find_include(importlib->name, NULL);
-    if(file_name) {
-        fd = open(file_name, O_RDONLY | O_BINARY );
-        free(file_name);
-    }else {
-        fd = open(importlib->name, O_RDONLY | O_BINARY );
-    }
+    fd = open_typelib(importlib->name);
+
+    /* widl extension: if importlib name has no .tlb extension, try using .tlb */
+    if (fd < 0 && !strendswith( importlib->name, ".tlb" ))
+        fd = open_typelib( strmake( "%s.tlb", importlib->name ));
 
     if(fd < 0)
-        error("Could not open importlib %s.\n", importlib->name);
+        error("Could not find importlib %s.\n", importlib->name);
 
-    tlb_read(fd, &magic, sizeof(magic));
+    size = lseek( fd, 0, SEEK_END );
+    data = xmalloc( size );
+    lseek( fd, 0, SEEK_SET );
+    if (read( fd, data, size) < size) error("error while reading importlib.\n");
+    close( fd );
 
-    switch(magic) {
-    case MSFT_MAGIC:
-        read_msft_importlib(importlib, fd);
-        break;
-    default:
-        error("Wrong or unsupported typelib magic %x\n", magic);
-    };
+    if (!memcmp( data, "MSFT", 4 ))
+        read_msft_importlib(importlib, data, size);
+    else if (!memcmp( data, "MZ", 2 ))
+        read_pe_importlib(importlib, data, size);
+    else
+        error("Wrong or unsupported typelib\n");
 
-    close(fd);
+    free( data );
 }
 
-void add_importlib(const char *name)
+void add_importlib(const char *name, typelib_t *typelib)
 {
     importlib_t *importlib;
 
